#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : embeddings
# @Time         : 2023/7/31 10:54
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/abetlen/llama-cpp-python/blob/main/llama_cpp/server/app.py

from meutils.pipe import *
from chatllm.schemas.embedding import EmbeddingRequest

from fastapi import Depends, FastAPI, APIRouter, Request, Response
from openai.types.create_embedding_response import CreateEmbeddingResponse, Embedding, Usage
from sentence_transformers import SentenceTransformer

router = APIRouter()

EMBEDDING_MODEL = os.getenv("EMBEDDING_MODEL", "/Users/betterme/PycharmProjects/AI/embeddings/m3e-small")
sentencetransformer = SentenceTransformer(model_name_or_path=EMBEDDING_MODEL)


@router.post("/embeddings", summary="Embedding")  # request.model_dump(exclude={"user"})
async def create_embeddings(request: EmbeddingRequest) -> CreateEmbeddingResponse:
    texts = request.input
    model = request.model  # todo: 支持多模型

    if isinstance(texts, str):
        texts = [texts]

    embeddings = sentencetransformer.encode(texts).tolist()
    data = [Embedding(index=i, embedding=e, object="embedding") for i, e in enumerate(embeddings)]

    usage = Usage(prompt_tokens=100 * len(texts), total_tokens=100 * len(texts))

    return CreateEmbeddingResponse(data=data, model=model, object="list", usage=usage)


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : st_chat
# @Time         : 2023/8/11 14:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import streamlit as st
# st.set_page_config('🔥ChatLLM', layout='wide', initial_sidebar_state='collapsed')



from chatllm.llmchain import init_cache
from chatllm.llmchain.applications import ChatFile
from chatllm.llmchain.document_loaders import FileLoader
from chatllm.llmchain.embeddings import OpenAIEmbeddings
from meutils.serving.streamlit import hide_st_style, st_chat_message, ChatMessage

from chatllm.llmchain.prompts.rag import template

hide_st_style()

init_cache(1)


@st.cache_resource(show_spinner=False)
def fn(file):
    docs = FileLoader(file, file.name).load_and_split()
    print(file.name, len(docs))

    chatfile = ChatFile(embeddings=OpenAIEmbeddings(chunk_size=20), prompt_template=template)
    chatfile.create_index(docs)

    return chatfile


chatfile = fn(open("东北证券股份有限公司合规手册（东证合规发〔2022〕25号 20221229）.pdf", 'rb'))

if __name__ == '__main__':
    col1, col2, col3, *_ = st.columns(3)
    with col1:
        st.image('规丞相.png', width=64)
    with col2:
        st.markdown('##### ')
        st.markdown('##### ')
        st.markdown('##### 规丞相驾到')

    st.markdown('> ⚠️“规丞相”仅供东北证券内部测试使用，所做回答不得用于东北证券官方回复。')

    st_chat_message(ChatMessage(avatar="规丞相.png", generator="欢迎来找**规丞相**，您有什么要咨询的吗❓"))

    for message in st.session_state.messages:  # 设计his
        st_chat_message(message)

    print(st.session_state.messages)

    prompt = st.chat_input("    🤔 你可以问我任何问题", key='xx')  # 最下面
    if prompt:
        with st.spinner('AI 🤔'):
            st_chat_message(ChatMessage(name="user", generator=prompt), is_history=True)

            st_chat_message(
                ChatMessage(
                    avatar="!.png",
                    generator="`大模型根据公司相关法规及提问语义做答，不作为合规管理官方回复。`"
                )
            )

            generator = chatfile.llm_qa(prompt, k=1)

            st_chat_message(ChatMessage(avatar="规丞相.png", generator=generator), is_history=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : app
# @Time         : 2023/5/26 14:59
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

# os.environ['LLM_MODEL'] = '/Users/betterme/PycharmProjects/AI/CHAT_MODEL/chatglm'
os.environ['DEBUG'] = '1'
os.environ['DB_URL'] = "mysql+pymysql://root:root123456@localhost/test"

from meutils.serving.fastapi import App

from chatllm.api.routes.api import router

app = App()
app.include_router(router)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : TextLoader
# @Time         : 2023/8/15 15:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from langchain.text_splitter import *
from langchain.document_loaders import TextLoader as _TextLoader
from langchain.document_loaders.base import BaseLoader
from langchain.schema import Document

from meutils.pipe import *
from meutils.fileparser import stream_parser


class TextLoader(_TextLoader):
    def __init__(
        self,
        # Union[str, bytes, bytearray, os.PathLike, io.BytesIO, io.TextIOBase, io.BufferedReader]
        file_path: Any,
        encoding: Optional[str] = None,
        autodetect_encoding: bool = False,
    ):
        """Initialize with file path."""
        self.file_path = file_path
        self.encoding = encoding
        self.autodetect_encoding = autodetect_encoding

    def load(self) -> List[Document]:
        if (
            isinstance(self.file_path, (str, os.PathLike))
            and len(self.file_path) < 256
            and Path(self.file_path).is_file()
        ):
            return _TextLoader(self.file_path, autodetect_encoding=True).load()

        filename, file_stream = stream_parser(self.file_path)
        if isinstance(file_stream, (bytes, bytearray)):
            file_stream = file_stream.decode()

        return [Document(page_content=file_stream, metadata={"source": filename})]


if __name__ == '__main__':
    print(TextLoader('pdf.py').load())
    print(TextLoader(open('pdf.py')).load())
    print(TextLoader(open('pdf.py').read()).load())
    print(TextLoader(open('pdf.py', 'rb')).load())
    print(TextLoader(open('pdf.py', 'rb').read()).load())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : hunyuan
# @Time         : 2023/10/9 14:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.decorators.retry import retrying

import hashlib
import hmac
from chatllm.schemas.openai_api_protocol import *

_SIGN_HOST = "hunyuan.cloud.tencent.com"
_SIGN_PATH = "hyllm/v1/chat/completions"
_URL = "https://hunyuan.cloud.tencent.com/hyllm/v1/chat/completions"


class HunYuanCompletion(object):

    @classmethod
    def create(
        cls,
        messages: List[Dict[str, Any]],  # [{'role': 'user', 'content': '讲个故事'}]
        **kwargs,
    ):
        logger.debug(kwargs)

        app_id, secret_id, secret_key = kwargs.pop('openai_api_key').split(':')

        if kwargs.get('stream'):
            return stream_create(app_id, secret_id, secret_key, messages, **kwargs)
        else:
            return create(app_id, secret_id, secret_key, messages, **kwargs)


@retrying
def create(appid, secretid, secretkey, messages, **kwargs):
    request = gen_param(appid, messages, secretid, **kwargs)
    signature = gen_signature(secretkey, gen_sign_params(request))
    # print(signature)
    headers = {
        "Content-Type": "application/json",
        "Authorization": str(signature)
    }
    # print('Input:\n{} | {} | {}'.format(_URL, headers, request))

    url = _URL
    resp = requests.post(url, headers=headers, json=request, stream=True)
    print(resp.text)
    _ = resp.json()
    _['model'] = 'hunyuan'
    return _


@retrying
def stream_create(appid, secretid, secretkey, messages, **kwargs):
    """
    {
        "req_id": "5803e3ca-a000-46e6-abb1-f98a775db161",
        "note": "以上内容为AI生成，不代表开发者立场，请勿删除或修改本标记",
        "choices": [
            {
                "finish_reason": "stop",
                "delta": {
                    "content": ""
                }
            }
        ],
        "created": "1696830392",
        "id": "test_query_id_68a2fcbb-4e78-4b8e-8817-e677e4197867",
        "usage": {
            "prompt_tokens": 3,
            "completion_tokens": 101,
            "total_tokens": 104
        }
    }
    """
    request = gen_param(appid, messages, secretid, **kwargs)
    signature = gen_signature(secretkey, gen_sign_params(request))
    # print(signature)
    headers = {
        "Content-Type": "application/json",
        "Authorization": str(signature)
    }
    url = _URL
    resp = requests.post(url, headers=headers, json=request, stream=True)
    # 如果使用出错请注意sse版本是否正确, pip3 install sseclient-py==1.7.2
    try_import('sseclient', pip_name='sseclient-py==1.7.2')
    import sseclient

    client = sseclient.SSEClient(resp)
    for event in client.events():
        if event.data:
            _ = json.loads(event.data)
            _['model'] = 'hunyuan'
            yield _


def gen_param(appid, messages, secretid, **kwargs):
    timestamp = int(time.time()) + 10000
    request = {
        "app_id": int(appid),
        "secret_id": secretid,
        "query_id": "test_query_id_" + str(uuid.uuid4()),
        "messages": messages,
        "temperature": kwargs.get("temperature", 0),
        "top_p": kwargs.get("top_p", 0.8),
        "stream": int(kwargs.get("stream", 0)),
        "timestamp": timestamp,
        "expired": timestamp + 24 * 60 * 60
    }
    return request


def gen_signature(secretkey, param):
    sort_dict = sorted(param.keys())
    sign_str = _SIGN_HOST + "/" + _SIGN_PATH + "?"
    for key in sort_dict:
        sign_str = sign_str + key + "=" + str(param[key]) + '&'
    sign_str = sign_str[:-1]
    # print(sign_str)
    hmacstr = hmac.new(secretkey.encode('utf-8'),
                       sign_str.encode('utf-8'), hashlib.sha1).digest()
    signature = base64.b64encode(hmacstr)
    signature = signature.decode('utf-8')
    return signature


def gen_sign_params(data):
    params = dict()
    params['app_id'] = data["app_id"]
    params['secret_id'] = data['secret_id']
    params['query_id'] = data['query_id']
    # float类型签名使用%g方式，浮点数字(根据值的大小采用%e或%f)
    params['temperature'] = '%g' % data['temperature']
    params['top_p'] = '%g' % data['top_p']
    params['stream'] = data["stream"]
    # 数组按照json结构拼接字符串
    message_str = ','.join(
        ['{{"role":"{}","content":"{}"}}'.format(message["role"], message["content"]) for message in data["messages"]])
    message_str = '[{}]'.format(message_str)
    # print(message_str)
    params['messages'] = message_str
    params['timestamp'] = str(data["timestamp"])
    params['expired'] = str(data["expired"])
    return params


if __name__ == '__main__':
    api_key = os.getenv('HUNYUAN_API_KEY')
    # print(HunYuanCompletion.create(messages=[{'role': 'user', 'content': '你是谁'}], api_key=api_key))
    for i in HunYuanCompletion.create(messages=[{'role': 'user', 'content': '你是谁'}], api_key=api_key, stream=True):
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : oneapi_test
# @Time         : 2023/12/8 09:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

from openai import OpenAI

completions = OpenAI(
    base_url=os.getenv('OPENAI_API_BASE'),
    max_retries=3
).chat.completions

_ = completions.create(
    messages=[{'role': 'user', 'content': '1+1'}],
    model='gpt-3.5-turbo',
    stream=False
)
print(_)


# "nohup python3 -m jupyterlab --ip 0.0.0.0 --port 39000 --no-browser --allow-root &"

# nohup huggingface-cli download --resume-download --local-dir-use-symlinks False infgrad/stella-large-zh-v2 --local-dir infgrad/stella-large-zh-v2 &


# nohup huggingface-cli download --resume-download --local-dir-use-symlinks False BAAI/bge-reranker-base --local-dir BAAI/bge-reranker-base &


# nohup huggingface-cli download --resume-download --local-dir-use-symlinks False deepseek-ai/deepseek-llm-7b-chat --local-dir deepseek-ai/deepseek-llm-7b-chat &


# export MODEL="01-ai/Yi-6B-200K"
# nohup huggingface-cli download --resume-download --local-dir-use-symlinks False $MODEL --local-dir $MODEL &



# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2023/12/21 17:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from urllib.parse import quote, unquote


_ = quote("http://150.109.20.187:39000/v1|sk-wewfUjhh5aMmLcGJE5653430C9Fb42F7A3885f166568Ed0f|x|\n\n[永远相信美好的事情即将发生](https://api.chatllm.vip/)")

print(_)


print(unquote(_))



from langchain.text_splitter import CharacterTextSplitter,RecursiveCharacterTextSplitter
import re
from typing import List
from configs.model_config import SENTENCE_SIZE


class ChineseTextSplitter(CharacterTextSplitter):
    def __init__(self, pdf: bool = False, sentence_size: int = SENTENCE_SIZE, **kwargs):
        super().__init__(**kwargs)
        self.pdf = pdf
        self.sentence_size = sentence_size

    def split_text1(self, text: str) -> List[str]:
        if self.pdf:
            text = re.sub(r"\n{3,}", "\n", text)
            text = re.sub('\s', ' ', text)
            text = text.replace("\n\n", "")
        sent_sep_pattern = re.compile('([﹒﹔﹖﹗．。！？]["’”」』]{0,2}|(?=["‘“「『]{1,2}|$))')  # del ：；
        sent_list = []
        for ele in sent_sep_pattern.split(text):
            if sent_sep_pattern.match(ele) and sent_list:
                sent_list[-1] += ele
            elif ele:
                sent_list.append(ele)
        return sent_list

    def split_text(self, text: str) -> List[str]:   ##此处需要进一步优化逻辑
        if self.pdf:
            text = re.sub(r"\n{3,}", r"\n", text)
            text = re.sub('\s', " ", text)
            text = re.sub("\n\n", "", text)

        text = re.sub(r'([;；.!?。！？\?])([^”’])', r"\1\n\2", text)  # 单字符断句符
        text = re.sub(r'(\.{6})([^"’”」』])', r"\1\n\2", text)  # 英文省略号
        text = re.sub(r'(\…{2})([^"’”」』])', r"\1\n\2", text)  # 中文省略号
        text = re.sub(r'([;；!?。！？\?]["’”」』]{0,2})([^;；!?，。！？\?])', r'\1\n\2', text)
        # 如果双引号前有终止符，那么双引号才是句子的终点，把分句符\n放到双引号后，注意前面的几句都小心保留了双引号
        text = text.rstrip()  # 段尾如果有多余的\n就去掉它
        # 很多规则中会考虑分号;，但是这里我把它忽略不计，破折号、英文双引号等同样忽略，需要的再做些简单调整即可。
        ls = [i for i in text.split("\n") if i]
        for ele in ls:
            if len(ele) > self.sentence_size:
                ele1 = re.sub(r'([,，.]["’”」』]{0,2})([^,，.])', r'\1\n\2', ele)
                ele1_ls = ele1.split("\n")
                for ele_ele1 in ele1_ls:
                    if len(ele_ele1) > self.sentence_size:
                        ele_ele2 = re.sub(r'([\n]{1,}| {2,}["’”」』]{0,2})([^\s])', r'\1\n\2', ele_ele1)
                        ele2_ls = ele_ele2.split("\n")
                        for ele_ele2 in ele2_ls:
                            if len(ele_ele2) > self.sentence_size:
                                ele_ele3 = re.sub('( ["’”」』]{0,2})([^ ])', r'\1\n\2', ele_ele2)
                                ele2_id = ele2_ls.index(ele_ele2)
                                ele2_ls = ele2_ls[:ele2_id] + [i for i in ele_ele3.split("\n") if i] + ele2_ls[
                                                                                                       ele2_id + 1:]
                        ele_id = ele1_ls.index(ele_ele1)
                        ele1_ls = ele1_ls[:ele_id] + [i for i in ele2_ls if i] + ele1_ls[ele_id + 1:]

                id = ls.index(ele)
                ls = ls[:id] + [i for i in ele1_ls if i] + ls[id + 1:]
        return ls
